﻿#include "mainpartframe.h"
#include <QMessageBox>
#include <QFile>
#include <QDebug>
#include <QTextStream>

#include "../PluginService/klinguist.h"
#include "../PluginService/ksystemenv.h"
#include "../PluginService/ksystemsetting_unit.h"
#include "../PluginService/kcommand.h"
#include "partman/partition_unit.h"
#include "../PluginService/kthread.h"
#include "delegate/customize_device_model.h"
#include "delegate/custom_partition_delegate.h"
#include "delegate/full_partition_delegate.h"
#include "../PluginService/ui_unit/arrowtooltip.h"


using namespace KServer;
namespace KInstaller {
using namespace Partman;
MainPartFrame::MainPartFrame(QWidget *parent) : MiddleFrameManager(parent),
    m_prepareFrame(new PrepareInstallFrame),
    m_partDelegate(new CustomPartitiondelegate(this)),
    m_quickDelegate(new FullPartitionDelegate(this)),
    m_partModel(new PartitionModel(this)),
    enumCheckBtn(InstallBtn::QuikPart),
    message(nullptr),
    m_partModelThread(new QThread)
{
    m_partModel->moveToThread(m_partModelThread);
    m_partModelThread->start();
    qDebug() << "m_partModelThread:" << m_partModelThread->currentThreadId();
        //准备安装提示窗口
    timer = new QTimer(this);
    timer->setInterval(100);
    connect(timer, &QTimer::timeout, this, &MainPartFrame::finishLoadDisk);
    timer->start();
    //    //设置鼠标等待状态
//    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));

    this->setObjectName("KPartition");
    initUI();
    initAllConnect();
    addStyleSheet();
    translateStr();
    scanDevices();
    m_nextBtn->setEnabled(false);
    this->setFocus();
}

MainPartFrame::~MainPartFrame()
{
    quitThreadRun(m_partModelThread);
}

void MainPartFrame::initUI()
{
    QGridLayout* gLayout = new QGridLayout;
    gLayout->setContentsMargins(0, 0, 0, 0);
    m_Widget->setLayout(gLayout);
    gLayout->setColumnStretch(0, 1);
    gLayout->setColumnStretch(1, 6);
    gLayout->setColumnStretch(2, 1);

    m_mainTitle = new QLabel(this);
    m_mainTitle->setObjectName("mainTitle");
    gLayout->addItem(new QSpacerItem(10, 40, QSizePolicy::Expanding, QSizePolicy::Preferred), 0, 1, 1, 1);
    gLayout->addWidget(m_mainTitle, 1, 1, 1, 1, Qt::AlignCenter | Qt::AlignTop);
    gLayout->addItem(new QSpacerItem(10, 10, QSizePolicy::Expanding, QSizePolicy::Preferred), 2, 1, 1, 1);

       //添加快速安装,并存安装和自定义安装
    m_fullInstallBtn = new QPushButton();
    m_fullInstallBtn->setDefault(1);
    m_fullInstallBtn->setCheckable(true);
    m_fullInstallBtn->setChecked(true);
    m_customInstallBtn = new QPushButton();
//    m_coexistInstallBtn = new QPushButton();
    //添加按钮之间的分割线
    QHBoxLayout *hBtnLayout = new QHBoxLayout(this);
    QFrame* installLine1 = new QFrame(this);
//    QFrame* installLine2 = new QFrame(this);

    installLine1->setObjectName(QStringLiteral("installLine1"));
//    installLine2->setObjectName(QStringLiteral("installLine2"));

    //设置按钮线形式
    installLine1->setFrameShape(QFrame::VLine);
//    installLine2->setFrameShape(QFrame::VLine);

    installLine1->setFrameShadow(QFrame::Sunken);
//    installLine2->setFrameShadow(QFrame::Sunken);

//    //设置线的样式
    installLine1->setMaximumWidth(1);
    installLine1->setMaximumHeight(15);
//    installLine2->setMaximumWidth(1);
//    installLine2->setMaximumHeight(15);

    //添加按钮和分割线到布局
    hBtnLayout->setSpacing(5);
    hBtnLayout->addWidget(m_fullInstallBtn);
    hBtnLayout->addWidget(installLine1);
//    hBtnLayout->addWidget(m_coexistInstallBtn);
//    hBtnLayout->addWidget(installLine2);
    hBtnLayout->addWidget(m_customInstallBtn);
    gLayout->addLayout(hBtnLayout, 3, 1, 1, 1, Qt::AlignCenter | Qt::AlignTop);
    gLayout->addItem(new QSpacerItem(10, 30, QSizePolicy::Expanding, QSizePolicy::Fixed), 4, 1, 1, 1);
//    gLayout->setRowStretch(4, 1);
    m_loadpic = new QLabel;
    m_loadpic->setObjectName("loadpic");
    m_loadpic->setPixmap(QPixmap(":/res/pic/loading/ukui-loading-0.svg").scaled(QSize(48, 48), Qt::KeepAspectRatio, Qt::SmoothTransformation));
    m_loadpic->setScaledContents(true);
//    m_loadpic->setAlignment(Qt::AlignCenter );
    gLayout->addWidget(m_loadpic, 5, 1, 2, 1, Qt::AlignCenter);

    m_fullInstallBtn->setObjectName("quikInstallBtn");
    m_customInstallBtn->setObjectName("customInstallBtn");
//    m_coexistInstallBtn->setObjectName("coexistInstallBtn");


    m_mainStackWidget = new QStackedWidget();

    m_fullPartFrame = new FullPartitionFrame(m_quickDelegate, this);
//    m_coexitPartFrame = new CoexistPartitionFrame(, this);
    m_customPartFrame = new CustomPartitionFrame(m_partDelegate, this);

    m_mainStackWidget->addWidget(m_fullPartFrame);
//    m_mainStackWidget->addWidget(m_coexitPartFrame);
    m_mainStackWidget->addWidget(m_customPartFrame);

    m_mainStackWidget->setCurrentWidget(m_fullPartFrame);
    this->setFocus();
    gLayout->addWidget(m_mainStackWidget, 5, 1, 1, 1);
    gLayout->setRowStretch(6, 1);
    qDebug() << Q_FUNC_INFO ;

}


void MainPartFrame::showCustomPartFrame()
{
    //m_quikPartFrame->releaseKeyboard();
    if(m_customPartFrame) {
        m_mainStackWidget->setCurrentWidget(m_customPartFrame);
        enumCheckBtn = InstallBtn::CustomPart;
        m_customInstallBtn->setCheckable(true);
        m_customInstallBtn->setChecked(true);
        m_fullInstallBtn->setChecked(false);
        getChoiceDisk(true);
//        m_coexistInstallBtn->setChecked(false);
    } else {
        qDebug() << "m_manualPartFrame:" << "not defined" ;
    }
}

void MainPartFrame::scanDevices()
{
    m_partModel->scanDevices();
}

void MainPartFrame::showQuikPartFrame()
{
    if(m_fullPartFrame) {
        if(m_quickDelegate->selectedDevices().length() > 0) {
            getChoiceDisk(true);
        } /*else {
            getChoiceDisk(false);
        }*/
        m_mainStackWidget->setCurrentWidget(m_fullPartFrame);
        enumCheckBtn = InstallBtn::QuikPart;
        m_customInstallBtn->setChecked(false);
        m_fullInstallBtn->setCheckable(true);
        m_fullInstallBtn->setChecked(true);
        this->setFocus();
//        m_coexistInstallBtn->setChecked(false);
    } else {
        qDebug() << "m_quikPartFrame:" << "not defined" ;
    }
}
void MainPartFrame::showCoexistFrame()
{
    if(m_coexitPartFrame) {
        m_mainStackWidget->setCurrentWidget(m_coexitPartFrame);
        enumCheckBtn = InstallBtn::CoexistPart;
        m_customInstallBtn->setChecked(false);
        m_fullInstallBtn->setChecked(false);
//        m_coexistInstallBtn->setCheckable(true);
//        m_coexistInstallBtn->setChecked(true);
    } else {
        qDebug() << "m_coexitPartFrame:" << "not defined" ;
    }
}

void MainPartFrame::setBtnEnable(bool bl)
{
    m_nextBtn->setEnabled(bl);
    m_fullInstallBtn->setEnabled(bl);
    m_customInstallBtn->setEnabled(bl);
}

void MainPartFrame::showCreateFrame()
{

}
void MainPartFrame::finishLoadDisk()
{

    setBtnEnable(false);
    qDebug() << "finishLoadDisk";
    var++;
    if(var > 7) {
        var = 0;
    }
    QString s = QString(":/res/pic/loading/ukui-loading-%1.svg").arg(var);

    m_loadpic->setPixmap(QPixmap(s).scaled(QSize(48, 48), Qt::KeepAspectRatio, Qt::SmoothTransformation));
    m_loadpic->setScaledContents(true);
    m_loadpic->update();
}

//void MainPartFrame::slotUpdateMainPartFrame(int nextPage)
//{

//}

void MainPartFrame::initAllConnect()
{
    connect(this, &MainPartFrame::signalFocus, this,&MainPartFrame::setFrameFocus);
    connect(m_fullPartFrame, &FullPartitionFrame::signalSeclectedListItem, this, &MainPartFrame::getChoiceDisk);
    connect(m_customPartFrame, &CustomPartitionFrame::signalNextBtn, this, &MainPartFrame::getChoiceDisk);
    connect(m_fullPartFrame, &FullPartitionFrame::enterpressed, this, &MainPartFrame::enterpressed);
    connect(m_customPartFrame, &CustomPartitionFrame::enterpressed, this, &MainPartFrame::enterpressed);
    connect(m_fullPartFrame, &FullPartitionFrame::backspacepressed, this, &MainPartFrame::backspacepressed);
    connect(m_customPartFrame, &CustomPartitionFrame::backspacepressed, this, &MainPartFrame::backspacepressed);
    connect(m_prepareFrame, &PrepareInstallFrame::backspacepressed, this, &MainPartFrame::backspacepressed);
    connect(this, &MainPartFrame::leftKeyPressed, m_fullPartFrame, &FullPartitionFrame::leftCliked);
    connect(this, &MainPartFrame::rightKeyPressed, m_fullPartFrame, &FullPartitionFrame::rightClicked);
    connect(m_fullPartFrame, &FullPartitionFrame::signalFinishedLoadDisk, [=]{
        m_loadpic->hide();
        m_fullPartFrame->m_encryptBox->show();
//        //恢复鼠标正常状态
//        QApplication::restoreOverrideCursor();
        setBtnEnable(true);
        timer->stop();

    });

    connect(m_partModel, &PartitionModel::deviceRefreshed, m_quickDelegate, &FullPartitionDelegate::onDeviceRefreshed);
    connect(m_partModel, &PartitionModel::deviceRefreshed, m_partDelegate, &CustomPartitiondelegate::onDeviceRefreshed);
//    connect(m_prepareFrame, &PrepareInstallFrame::signalNextStartInstalling, [=]{emit signalStartInstall();});
    connect(m_fullInstallBtn, &QPushButton::clicked, this, &MainPartFrame::showQuikPartFrame);
    connect(m_customInstallBtn, &QPushButton::clicked, this, &MainPartFrame::showCustomPartFrame);

    connect(m_customPartFrame, &CustomPartitionFrame::signalCreatePartTable, m_partDelegate, &CustomPartitiondelegate::createDeviceTable);
//    qDebug() << "signalquickpart:" << connect(m_prepareFrame, &PrepareInstallFrame::signalQuickPart, m_partModel, &PartitionModel::quickPart);
    connect(m_prepareFrame, &PrepareInstallFrame::signalQuickPart, [=]{
        emit signalStartInstall();
        m_partModel->fullPart();
    });
    connect(m_prepareFrame, &PrepareInstallFrame::signalCustomPart, [=] {
        emit signalStartInstall();
//        //设置鼠标等待状态
//        QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
        m_partModel->customPart(m_partDelegate->operations());
//        emit m_partModel->signalInstallCustomPart(m_partDelegate->operations());
            });
    qDebug() << "PartitionModel::customPartDone" << connect(m_partModel, &PartitionModel::customPartDone, [=](int execcode) {
        if(execcode == 0) {
            emit signalInstalScriptlFinished();
        } else
            emit signalInstallError(QString::number(execcode));
    });
    qDebug() << "PartitionModel::autoPartDone" << connect(m_partModel, &PartitionModel::fullPartDone, [=](int execcode) {
        if(execcode == 0) {
            emit signalInstalScriptlFinished();
        } else
            emit signalInstallError(QString::number(execcode));});

}

void MainPartFrame::clickNextButton()
{
    if(message != nullptr) {
        message->close();
    }
    message = new MessageBox;
    m_prepareFrame->setInstallBtn(enumCheckBtn);

    switch (enumCheckBtn) {
    case InstallBtn::QuikPart:
        if(!m_quickDelegate->DiskSizeValidate(m_fullPartFrame->m_devPath)) {
            message->setMessageInfo(tr("Disk must be greater than 50G or No disk selected"));
            message->setCancleHidden();
            int exec = message->exec();
            emit signalFocus();
            if(exec == QDialog::Accepted ) {

                return;
            } else if(message->execid == 1){
                emit signalFocus();
                return;
            }
        }
//        //提示为解决磁盘当前的分区表类型和引导不一致导致不能 正常安装问题
//        if(!checkLoaderAndPartitionTable(m_quickDelegate->findDevice(m_quikPartFrame->m_devPath))) {
//            message->setMessageInfo(tr("BootLoader method inconsistent with the disk partition table type."));
//            if(message->exec() == QDialog::Rejected && message->execid == 1) {
//                emit signalFocus();
//                return;
//            }
//        }

        m_prepareFrame->setMainTitleStr(tr("Confirm Full Installation"));//快速安装
        m_prepareFrame->m_tipCheck->setText(tr("Formatted the whole disk"));
        if(!m_quickDelegate->isMBRPreferredDe()) {
            partStr = QString(tr("%1 设备上第1分区为 EFI, 将设为 vfat;\n"
                                 "%1 设备上第2分区为 boot, 将设为 ext4;\n"
                                 "%1 设备上第3分区为 /, 将设为 ext4;\n"
                                 "%1 设备上第4分区为 backup, 将设为 ext4;\n"
                                 "%1 设备上第5分区为 data, 将设为 ext4;\n"
                                 "%1 设备上第6分区为 swap, 将设为 swap;\n")).arg(m_fullPartFrame->m_devPath);
        } else {
            partStr = QString(tr("%1 设备上第1分区为 boot, 将设为 ext4;\n"
                                 "%1 设备上第2分区为 extend;\n"
                                 "%1 设备上第5分区为 /, 将设为 ext4;\n"
                                 "%1 设备上第6分区为 backup, 将设为 ext4;\n"
                                 "%1 设备上第7分区为 data, 将设为 ext4;\n"
                                 "%1 设备上第8分区为 swap, 将设为 swap;\n")).arg(m_fullPartFrame->m_devPath);
        }

        m_prepareFrame->setPartInfo(partStr);
        writeSettingIni(InstallBtn::QuikPart);
        break;
    case InstallBtn::CoexistPart:
        m_prepareFrame->setMainTitleStr(tr("Coexist Install"));//并存安装
        break;
    case InstallBtn::CustomPart:
        if(!validatePartition()) {
            message->setMessageInfo(validateMessage.left(validateMessage.size() - 1));
            message->setCancleHidden();
            if (message->exec() == QDialog::Accepted ){
                emit signalFocus();
                return;
            } else if(message->execid == 1){
                emit signalFocus();
                return;
            }
        } else if(validatePartition() == -1) {
            message->setMessageInfo(validateMessage.left(validateMessage.size() - 1));
            if(message->exec() == QDialog::Rejected  && message->execid == 1){
                emit signalFocus();
                return;
            }
        }
        //提示为解决磁盘当前的分区表类型和引导不一致导致不能 正常安装问题
        if(!checkLoaderAndPartitionTable(m_partDelegate->findDevice(m_customPartFrame->m_bootCombox->currentText().split(" ").at(0)))) {
            message->setMessageInfo(checkLoaderAndPartitionTableStr());
            if(message->exec() == QDialog::Rejected && message->execid == 1) {
                emit signalFocus();
                return;
            }
        }
        m_prepareFrame->setMainTitleStr(tr("Confirm Custom Installation"));//确认自定义安装

        /*
         * 如果您继续，以下所列出的修改内容将会写入到磁盘中。或者，您也可以手动来进行其它修改。
         * 警告：任何已删除和将要格式化的分区上的数据都将被摧毁。
         */
        partStr = m_partDelegate->comboxOperatorPartitions();
        m_prepareFrame->setPartInfo(partStr);
        m_prepareFrame->m_tipCheck->setText(tr("Confirm the above operations"));
        m_partDelegate->writeMountOperate();
        writeSettingIni(InstallBtn::CustomPart);

        break;

    }

    emit signalStackFrameManager((QWidget*)m_prepareFrame);
}

//bug 51780 add message
QString MainPartFrame::checkLoaderAndPartitionTableStr()
{
    bool table = checkLoaderAndPartitionTable(m_partDelegate->findDevice(m_customPartFrame->m_bootCombox->currentText().split(" ").at(0)));
    bool start = checkSpecialModel();

    QString info;
    if(start != table) {
        if(start == 1) {
            info = QString(tr("BootLoader method %1 inconsistent with the disk partition table \n"
                              "type %2.").arg("UEFI").arg("MSDos"));
        } else {
            info = QString(tr("BootLoader method %1 inconsistent with the disk partition table \n"
                              "type %2, cannot have efi partition.").arg("legacy").arg("GPT"));
        }
    } else {
        if(start == 0) {
            info = QString(tr("BootLoader method %1 inconsistent with the disk partition table \n"
                              "type %2, cannot have efi partition.").arg("legacy").arg("MSDos"));
        }
    }
    return info;

}


//返回值为1：检测通过，0：检测不通过，-1：检测不存在可继续操作
int MainPartFrame::validatePartition()
{
    validateMessage.clear();
    m_validateStates = m_partDelegate->validate();
    if(m_validateStates.isEmpty()) {
        return 1;
    } else {
       for(ValidateState state : m_validateStates) {
           qDebug() <<"MainPartFrame::validatePartition:" << state->state();
           switch (state->state()) {
           case ValidateState::InvalidId:
               validateMessage.append(tr("InvalidId\n"));
               break;
           case ValidateState::BootFsInvalid:
               validateMessage.append(tr("Boot filesystem invalid\n"));
               return 0;
           case ValidateState::BootPartNumberInvalid:
               validateMessage.append(tr("Boot partition not in the first partition\n"));
               return 0;
           case ValidateState::BootTooSmall:
               validateMessage.append(tr("Boot partition too small\n"));
               return 0;
           case ValidateState::BootMissing:
               validateMessage.append(tr("No boot partition\n"));
               return 0;
           case ValidateState::EfiMissing:
               validateMessage.append(tr("No Efi partition\n"));
               return 0;
           case ValidateState::EfiTooSmall:
               validateMessage.append(tr("Efi partition too small\n"));
               return 0;
           case ValidateState::EfiTooMany:
               validateMessage.append(tr("Only one EFI partition is allowed\n"));
               return 0;
           case ValidateState::EfiPartNumberInvalid:
               validateMessage.append(tr("Efi partition number invalid\n"));
               break;
           case ValidateState::RootMissing:
               validateMessage.append(tr("No root partition\n"));
               return 0;
           case ValidateState::RootTooSmall:
               validateMessage.append(tr("Root partition size is greater than 15GiB,\n"
                                         "but Huawei machines require greater than 25GiB.\n"));//根分区大小大于15GiB，华为机器要求大于25GiB
               return 0;
           case ValidateState::PartitionTooSmall:
               validateMessage.append(tr("Partition too small\n"));
               break;
           case ValidateState::BackUpPartitionTooSmall:
               validateMessage.append(tr("BackUp partition too small\n"));
               break;
           case ValidateState::RepeatedMountPoint:
               validateMessage.append(tr("Repeated mountpoint\n"));
               return 0;
           case ValidateState::DataORBackupPartition:
               validateMessage.append(tr("No backup partition\n"));
               return -1;//
//           case ValidateState::HUAWEIKirin990:
//               validateMessage.append(tr("HUAWEI Kirin 990 format EFI partition to fat16"));
//               return 0;
//           case ValidateState::MipsBootExt3:
//               validateMessage.append(tr("Mips format Boot partition to Ext3\n"));
//               return 0;
           case ValidateState::MipsNotSupportEFI:
               validateMessage.append(tr("This machine not support EFI partition\n"));
               return 0;
           case ValidateState::InvalidFS:
               validateMessage.append(tr("The filesystem type FAT16 or FAT32 is not fully-function filesystem,\n"
                                        "except EFI partition, other partition not proposed"));
               return 0;
           default:
               break;
           }
        }
        return true;
    }
}

void MainPartFrame::addStyleSheet()
{
    QFile file(":/res/qss/KPartition.css");
    qDebug() << file.open(QFile::ReadOnly);
    QTextStream filetext(&file);
    QString stylesheet = filetext.readAll();
    file.close();

    this->setStyleSheet(stylesheet);
}

void MainPartFrame::changeEvent(QEvent *event)
{
    if(event->type() == QEvent::LanguageChange) {
        translateStr();
    } else {
        QWidget::changeEvent(event);
    }
}

void MainPartFrame::translateStr()
{
    m_mainTitle->setText(tr("Choose Installation Method"));
    m_fullInstallBtn->setText(tr("Full install"));
    m_customInstallBtn->setText(tr("Custom install"));
//    m_coexistInstallBtn->setText(tr("Coexist install"));
    m_nextBtn->setText(tr("Next"));
}

void MainPartFrame::writeSettingIni(InstallBtn btn)
{
    if(btn == InstallBtn::QuikPart) {
        WriteSettingToIni("config", "devpath", m_fullPartFrame->m_devPath);
        WriteSettingToIni("config", "automatic-installation", QString::number(1));
    } else if(btn == InstallBtn::CustomPart) {
        WriteSettingToIni("config", "automatic-installation", QString::number(0));
        WriteSettingToIni("custompartition", "partitions", m_partDelegate->iniPartitions);
        WriteSettingToIni("custompartition", "bootloader", m_partDelegate->getBootLoadPath());

    }

}


void MainPartFrame::getChoiceDisk(bool flag)
{
    if(flag) {
        m_nextBtn->setEnabled(true);
        m_nextBtn->setStyleSheet("background: rgb(29, 142, 255);");
        this->setFocus();
    } else {
        m_nextBtn->setEnabled(false);
        m_nextBtn->setStyleSheet("color: rgba(255, 255, 255, 0.4);background:#2D6BAA;");
    }
}
void MainPartFrame::closeArrowWidget()
{

}




void MainPartFrame::keyPressEvent(QKeyEvent *event)
{
    this->setFocus();
    qDebug() << Q_FUNC_INFO << "keyPressEvent";
    if (event->key() == Qt::Key_Left) {
        qDebug() << "left key pressed!";
        emit leftKeyPressed();
    }
    else if (event->key() == Qt::Key_Right){
        qDebug() << "right key pressed!";
        emit rightKeyPressed();
    }
    else if(event->key() == Qt::Key_Enter || event->key() == Qt::Key_Return)
    {
        emit enterpressed();
    }
    else if(event->key() == Qt::Key_Backspace)
    {
        emit backspacepressed();
    }
    else if (event->key() == Qt::Key_Control || event->key() == Qt::Key_Escape) {
        this->setFocus();
    }
    else
        QWidget::keyPressEvent(event);
}


void MainPartFrame::setFrameFocus()
{
    qDebug() << Q_FUNC_INFO;
    this->setFocus();
//    switch (enumCheckBtn) {
//    case InstallBtn::QuikPart:
//        m_quikPartFrame->setFocus();
//        break;
//    case InstallBtn::CustomPart:
//        m_customPartFrame->setFocus();
//        break;
//    case InstallBtn::CoexistPart:
//        break;
//    default:
//        this->setFocus();
//    }
}

}
