﻿#include "userframe.h"
#include <pwquality.h>
#include <QApplication>
#include <clocale>
#include <QDebug>
#include <QKeyEvent>
#include <QButtonGroup>
#include <QHBoxLayout>
#include <QtDBus>
#include <QRegExp>

#include "locale.h"
#include "libintl.h"
#include <QDBusMessage>
#include <QDBusArgument>
#include <QSizePolicy>
#include "../PluginService/ksystemenv.h"
#include "../PluginService/ksystemsetting_unit.h"
#include "../PluginService/kcommand.h"
#include "../PluginService/kdir.h"
#include "../PluginService/sysInfo/kpasswordcheck.h"

namespace KInstaller{

using namespace KServer;


UserFrame::UserFrame(QWidget *parent) : MiddleFrameManager(parent),
    userSetFinished(false),
    hostSetFinished(false),
    pwdSetFinished(false),
    cpwdSetFinished(false),
    errtip(""),
    m_autoLoginFlag(false),
    w(nullptr),
    tooltipStr(""),
    m_LabelState(LabelState::unkown),
    m_flag(true),
    blfinger(false)
{
    this->setObjectName("UserFrame");
    blfinger = findFingerPrintDevice();
    m_nextBtn->setEnabled(false);
    initUI();
    initAllConnect();
    addStyleSheet();
    showToolTip();
    setBoardTabOrder();
    KServer::KPasswordCheck::init();
    this->setFocus();

}

UserFrame::~UserFrame()
{
    KServer::KPasswordCheck::getInstance()->releasePWquality();
}

void UserFrame::initUI()
{
    QGridLayout *usersetupLayout = new QGridLayout();
//    m_Widget->setLayout(usersetupLayout);
    m_Widget->setLayout(usersetupLayout);
    usersetupLayout->setContentsMargins(0, 0, 0, 0);
    usersetupLayout->setSpacing(16);

    usersetupLayout->setColumnStretch(0, 3);
    usersetupLayout->setColumnStretch(1, 2);
    usersetupLayout->setColumnStretch(2, 3);

    m_mainTitle = new QLabel();
    m_mainTitle->setObjectName("mainTitle");

    usersetupLayout->addItem(new QSpacerItem(10, 40, QSizePolicy::Expanding, QSizePolicy::Preferred), 0, 1, 1, 1);
    usersetupLayout->addWidget(m_mainTitle, 1, 1, 1, 1, Qt::AlignCenter | Qt::AlignTop);
    usersetupLayout->addItem(new QSpacerItem(10, 34, QSizePolicy::Expanding, QSizePolicy::Preferred), 2, 1, 1, 1);

    //用户名框设置
    QHBoxLayout* hlayout1 = new QHBoxLayout;
    m_username = new CLineEditLabel("", QIcon(":/res/svg/user.svg"), this);
    m_username->setObjectName("username");
    m_userCheck = new QLabel(this);
    m_userCheck->setObjectName("checkpng");
    hlayout1->addLayout(m_username->layout);
//    hlayout1->addWidget(m_userCheck);

    //主机名框设置
    QHBoxLayout* hlayout2 = new QHBoxLayout;
    m_hostname = new CLineEditLabel("", QIcon(":/res/svg/computer.svg"), this);
    m_hostname->setObjectName("hostname");
    m_hostCheck = new QLabel(this);
    m_hostCheck->setObjectName("checkpng");
    hlayout2->addLayout(m_hostname->layout);
//    hlayout2->addWidget(m_hostCheck);

    //密码框设置
    QHBoxLayout* hlayout3 = new QHBoxLayout;
    m_passwd = new CLineEditLabel("", QIcon(":/res/svg/passwd.svg"), this);
    m_passwd->setObjectName("passwd");
    m_passwd->setTrailingPng(QIcon(":/res/svg/code.svg"));
    m_passwd->m_lineEdit->setEchoMode(QLineEdit::Password);
    m_passwd->setBLToolTip(true);

    QVBoxLayout* toolTiplayout = new QVBoxLayout();
    toolTiplayout->setContentsMargins(0, 0, 0, 0);
//    toolTiplayout->setSpacing(0);
//    toolTiplayout->setStretch(1, 100);
    //删除密码复杂度气泡提示 2021-4-20
//    w = new ArrowWidget(this);
//    toolTiplayout->addWidget(w, 0, Qt::AlignLeft | Qt::AlignTop);
//    toolTiplayout->addStretch();


    m_pwdCheck = new QLabel(this);
    m_pwdCheck->setObjectName("checkpng");
    hlayout3->addLayout(m_passwd->layout);
//    hlayout3->addWidget(m_pwdCheck);

    //确认密码框设置
    QHBoxLayout* hlayout4 = new QHBoxLayout;
    m_confirm_passwd = new CLineEditLabel("", QIcon(":/res/svg/confirm_passwd.svg"), this);
    m_confirm_passwd->m_lineEdit->setEchoMode(QLineEdit::Password);
    m_confirm_passwd->setObjectName("confirm_passwd");
    m_confirm_passwd->setTrailingPng(QIcon(":/res/svg/code.svg"));
    m_cpwdCheck = new QLabel(this);
    m_cpwdCheck->setObjectName("checkpng");
    hlayout4->addLayout(m_confirm_passwd->layout);
//    hlayout4->addWidget(m_cpwdCheck);

    usersetupLayout->addLayout(hlayout1, 3, 1, 1, 1, Qt::AlignCenter);
    usersetupLayout->addLayout(hlayout2, 4, 1, 1, 1, Qt::AlignCenter);
    usersetupLayout->addLayout(hlayout3, 5, 1, 1, 1, Qt::AlignCenter);
    usersetupLayout->addLayout(hlayout4, 6, 1, 1, 1, Qt::AlignCenter);
    usersetupLayout->addWidget(m_userCheck, 3, 2, 1, 1, Qt::AlignLeft);
    usersetupLayout->addWidget(m_hostCheck, 4, 2, 1, 1, Qt::AlignLeft);
    usersetupLayout->addWidget(m_pwdCheck, 5, 2, 1, 1, Qt::AlignLeft);
    usersetupLayout->addLayout(toolTiplayout, 5, 2, 2, 1, Qt::AlignLeft | Qt::AlignTop);
    usersetupLayout->addWidget(m_cpwdCheck, 6, 2, 1, 1, Qt::AlignLeft);


    m_cpwdErrorTip = new QLabel;
    m_cpwdErrorTip->setObjectName("cpwdErrorTip");
    m_cpwdErrorTip->setAlignment(Qt::AlignCenter);
    usersetupLayout->addWidget(m_cpwdErrorTip, 7, 0, 1, 3, Qt::AlignCenter);

    pwLoginCKBox = new QCheckBox();
    pwLoginCKBox->setChecked(false);
    usersetupLayout->addWidget(pwLoginCKBox, 8, 1, Qt::AlignLeft);
    usersetupLayout->setRowStretch(9,1);
    fingerprintpng = new QLabel;
    fingerprinttip = new QLabel;
    if(blfinger) {
        fingerprintpng->setObjectName("fingerpng");
        fingerprintpng->setPixmap(QPixmap(":/res/svg/finger.png").scaled(16, 16));
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy.setHeightForWidth(fingerprintpng->sizePolicy().hasHeightForWidth());
        sizePolicy.setWidthForHeight(fingerprintpng->sizePolicy().hasWidthForHeight());
        sizePolicy.setVerticalStretch(0);
        fingerprintpng->setSizePolicy(sizePolicy);


        fingerprinttip->setObjectName("fingertip");
        fingerprintpng->setContentsMargins(3, 3, 3, 3);
        QHBoxLayout* hlay = new QHBoxLayout;
        hlay->setSpacing(8);
        hlay->addWidget(fingerprintpng, Qt::AlignRight | Qt::AlignTop);
        fingerprintpng->setAlignment(Qt::AlignTop);
        hlay->addWidget(fingerprinttip, Qt::AlignLeft);
        fingerprinttip->setAlignment(Qt::AlignTop);
        hlay->addItem(new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Expanding));
        usersetupLayout->addLayout(hlay, 10, 1, 1, 2, Qt::AlignLeft | Qt::AlignTop);
    }

    usersetupLayout->addItem(new QSpacerItem(10, 30, QSizePolicy::Expanding, QSizePolicy::Preferred), 11, 1, 1, 1);
    translateStr();
}

bool UserFrame::findFingerPrintDevice()
{
//    QDBusInterface iface("org.ukui.Biometric",
//                         "/org/ukui/Biometric",
//                         "org.ukui.Biometric",
//                         QDBusConnection::systemBus());
//    QDBusReply<QVariant> ret = iface.call("GetDevList");

    QDBusMessage m = QDBusMessage::createMethodCall("org.ukui.Biometric",
                         "/org/ukui/Biometric",
                         "org.ukui.Biometric",
                         "GetDevList");

    int count = 0;
    QDBusMessage res = QDBusConnection::systemBus().call(m);
    if(res.type() == QDBusMessage::ReplyMessage) {
        count = res.arguments().takeFirst().toInt();
    } else {
        return false;
    }
    if(count > 0) {
        return true;
    } else
        return false;

}

void UserFrame::setBoardTabOrder()
{
    QWidget::setTabOrder(m_username, m_hostname);
    QWidget::setTabOrder(m_hostname, m_passwd);
    QWidget::setTabOrder(m_passwd, m_confirm_passwd);
    QWidget::setTabOrder(m_confirm_passwd, m_nextBtn);
}

void UserFrame::initAllConnect()
{
    QObject::connect(m_username->m_lineEdit, &QLineEdit::textChanged, this, &UserFrame::setComputerText);

    QObject::connect(m_username->m_lineEdit, &QLineEdit::textChanged, this, &UserFrame::checkUserName);
    QObject::connect(m_hostname->m_lineEdit, &QLineEdit::textChanged, this, &UserFrame::checkHostName);
    QObject::connect(m_passwd->m_lineEdit, &QLineEdit::textChanged, this, &UserFrame::checkPasswd);
    QObject::connect(m_confirm_passwd->m_lineEdit, &QLineEdit::textChanged, this, &UserFrame::checkCPasswd);
    connect(this, &UserFrame::signalConfirmPasswd, [=]{
        QString pwd = m_passwd->getLineEditText();
        QString cpwd = m_confirm_passwd->getLineEditText();
        if(!cpwd.isEmpty())
          recheckCPasswd(pwd, cpwd);
    });

    QObject::connect(pwLoginCKBox, &QCheckBox::clicked, this, &UserFrame::getAutoLogin);

}

void UserFrame::checkUserName(QString username)
{
    m_state = KCheckUserName(username);
    qDebug() << getErrorMsg() << "1";
    m_LabelState = LabelState::Username;
    if(m_state == CheckState::OK) {
        userSetFinished = true;
        m_flag = true;
    } else {
        userSetFinished = false;
        m_flag = false;
    }
    translateStr();
    setNextBtnStyle();

}

void UserFrame::checkHostName(QString hostname)
{
    m_state = KCheckHostName(hostname);
    qDebug() << getErrorMsg() << "2";
    m_LabelState = LabelState::Hostname;
    if(m_state == CheckState::OK) {
        hostSetFinished = true;
        m_flag = true;
    } else {
        hostSetFinished = false;
        m_flag = false;
    }
    translateStr();
    setNextBtnStyle();
}

void UserFrame::checkPasswd(QString pwd)
{
//    w->show();
    m_LabelState = LabelState::Pwd;
    m_state = KCheckPassWD(pwd, errstr, m_username->getLineEditText());
    if(m_state == CheckState::OK) {
        pwdSetFinished = true;
        errtip = "";
        m_flag = true;
        m_passwd->m_lineEdit->setStyleSheet("QLineEdit{border: 0px solid #5BADFF; }QLineEdit:hover { background: rgba(0, 0, 0, 0.15);border: 2px solid #5BADFF;}");

    } else {
        pwdSetFinished = false;
        errtip = errstr;
        m_passwd->m_lineEdit->setStyleSheet("QLineEdit{border-color: red }QLineEdit:hover { background: rgba(0, 0, 0, 0.15);border: 2px solid #5BADFF;}");
        m_flag = false;
    }
    translateStr();
    emit signalConfirmPasswd();
    setNextBtnStyle();
}

void UserFrame::checkCPasswd(QString cpwd)
{
    QString pwd = m_passwd->getLineEditText();
    m_state = KCheckPassWD(pwd, errstr, m_username->getLineEditText());
    if(m_state != CheckState::OK) {
        pwdSetFinished = false;
        errtip = errstr;
        m_passwd->m_lineEdit->setStyleSheet("QLineEdit{border-color: red }QLineEdit:hover { background: rgba(0, 0, 0, 0.15);border: 2px solid #5BADFF;}");
        m_flag = false;
        m_LabelState = LabelState::Pwd;
        translateStr();
        return;
    }
    m_LabelState = LabelState::CPwd;
    if(pwd.isEmpty() || (pwd != cpwd)) {
        m_flag = false;
        translateStr();
    }
    recheckCPasswd(pwd, cpwd);
    setNextBtnStyle();

}

void UserFrame::recheckCPasswd(QString pwd, QString cpwd)
{
    m_LabelState = LabelState::CPwd;
    if(pwd == cpwd && !pwd.isEmpty()) {
       cpwdSetFinished = true;
       m_flag = true;
    } else {
        cpwdSetFinished = false;
        m_flag = false;
    }
    translateStr();

}

void UserFrame::getAutoLogin()
{
    if(pwLoginCKBox->isChecked()) {
        m_autoLoginFlag = true;
    } else {
        m_autoLoginFlag = false;
    }
}

void UserFrame::markLineEditCorrect(LabelState state, bool flag)
{
    switch (state) {
    case LabelState::Username:
        if(flag) {
            m_userCheck->setPixmap(QPixmap(":/res/svg/correct.svg"));
            m_cpwdErrorTip->setText("");
        } else {
            m_userCheck->setPixmap(QPixmap(":/res/svg/error.svg"));
            m_cpwdErrorTip->setText(tr("Your username only letters,numbers,underscore and hyphen are allowed, no more than 32 bits in length.\n"
                                       "but start with a lowercase letter"));
        }
        break;
    case LabelState::Hostname:
        if(flag) {
            m_hostCheck->setPixmap(QPixmap(":/res/svg/correct.svg"));
            m_cpwdErrorTip->setText("");
        } else {
            m_hostCheck->setPixmap(QPixmap(":/res/svg/error.svg"));
            m_cpwdErrorTip->setText(tr("Your hostname only letters,numbers,underscore and hyphen are allowed, no more than 64 bits in length."));
        }
        break;
    case LabelState::Pwd:
        if(flag) {
//            w->hide();
            m_pwdCheck->setPixmap(QPixmap(":/res/svg/correct.svg"));
            m_cpwdErrorTip->setText("");
        } else {
            m_pwdCheck->setPixmap(QPixmap(":/res/svg/error.svg"));
            m_cpwdErrorTip->setText(errstr);
        }
        break;
    case LabelState::CPwd:
        if(flag) {
            m_cpwdCheck->setPixmap(QPixmap(":/res/svg/correct.svg"));
            m_cpwdErrorTip->setText("");
        } else {
            m_cpwdCheck->setPixmap(QPixmap(":/res/svg/error.svg"));
            m_cpwdErrorTip->setText(tr("Two password entries are inconsistent!"));
        }
        break;
    default:

        break;
    }
}

void UserFrame::addStyleSheet()
{
    QFile file(":/res/qss/KUserRegister.css");
    file.open(QFile::ReadOnly);
    QTextStream filetext(&file);
    QString stylesheet = QString("");
    stylesheet = filetext.readAll();
    file.close();

    this->setStyleSheet(stylesheet);
}

void UserFrame::setComputerText()
{
    QString path = QString("");
    QString output = QString("");
    QString error = QString("");
    int excid = 0;
    QString txt = ("[\\ ,\\.,\\*,\\&,\\\\,\\/,\\?,\\|,\\:,\\<,\\>,\"]");
    QRegExp reg(txt);
    KServer::KCommand::getInstance()->RunScripCommand("dmidecode", {"-s", "system-manufacturer"}, path, output, error, excid);
    output = output.toLower();
    if(output.contains("lenovo") || output.contains("ibm")) {
        KServer::KCommand::getInstance()->RunScripCommand("dmidecode", {"-s", "system-version"}, path, output, error, excid);
    } else
        KServer::KCommand::getInstance()->RunScripCommand("dmidecode", {"-s", "system-product-name"}, path, output, error, excid);
    if(output != "" && !output.contains("N/A") && !output.contains("loongson", Qt::CaseInsensitive)) {
//        m_hostname->setLineEditText(m_username->getLineEditText() + "-" + output.remove(QRegExp("\\s"));
//        output = "hdha da";
        m_hostname->setLineEditText(m_username->getLineEditText() + "-" + output.simplified().replace(reg, "-"));
    } else {
        m_hostname->setLineEditText(m_username->getLineEditText() + "-PC");
    }
}

void UserFrame::translateStr()
{
    m_mainTitle->setText(tr("Create User"));
    m_username->setPlaceholderText(tr("username"));
    m_hostname->setPlaceholderText(tr("hostname"));
    m_passwd->setPlaceholderText(tr("new password"));
    m_confirm_passwd->setPlaceholderText(tr("enter the password again"));
    m_nextBtn->setText(tr("Next"));
    pwLoginCKBox->setText(tr("Automatic login on boot"));
//    m_cpwdErrorTip->clear();
//    m_cpwdErrorTip->update();
//    tooltipStr = tr("Password length cannot be less than 6 characters\nPassword must have two different characters");
    fingerprinttip->setText(tr("Detecte fingerprint device.\n "
                               "Fingerprint entry can be done through [biometric management tool]."));
    markLineEditCorrect(m_LabelState, m_flag);
}

void UserFrame::changeEvent(QEvent *event)
{
    if(event->type() == QEvent::LanguageChange) {

        translateStr();
//        if(w != nullptr) {
//            w->setText(tooltipStr);
//        }

    } else {
        QWidget::changeEvent(event);
    }
}

QString UserFrame::getErrorMsg()
{
    QString errorstr;
    switch (m_state) {
    case CheckState::OK:
        errorstr = QObject::tr("good");
        return errorstr;
    case CheckState::EmptyError:
        errorstr = QObject::tr("Is empty");
        return errorstr;
    case CheckState::FirstCharError:
        errorstr = QObject::tr("The first character is wrong");
        return errorstr;
    case CheckState::InvalidCharError:
        errorstr = QObject::tr("Invalid character error");
        return errorstr;
    case CheckState::TooLongError:
        errorstr = QObject::tr("Too long");
        return errorstr;
    case CheckState::TooShortError:
        errorstr = QObject::tr("Too short");
        return errorstr;
    case CheckState::NoNumberError:
        errorstr = QObject::tr("No number error");
        return errorstr;
    case CheckState::NoLowerCharError:
        errorstr = QObject::tr("No lowercase letter errors");
        return errorstr;
    case CheckState::NoUpperCharError:
        errorstr = QObject::tr("No capital letter errors");
        return errorstr;
    case CheckState::NoSpecialCharError:
        errorstr = QObject::tr("No special character error");
        return errorstr;
    case CheckState::TooWeakError:
        errorstr = QObject::tr("Too weak");
        return errorstr;
    default:
        break;
    }
    return errorstr;
}

void UserFrame::clickNextButton()
{

    if(userSetFinished && hostSetFinished && pwdSetFinished && cpwdSetFinished) {
//        if(w != nullptr) {
//            w->close();
//        }
        emit signalKeyboradClose();
        writeSettingIni();
        emit signalStackPageChanged(1);
    }
}

void UserFrame::writeSettingIni()
{
    WriteSettingToIni("config", "username", m_username->getLineEditText());
    WriteSettingToIni("config", "hostname", m_hostname->getLineEditText());
    WriteSettingToIni("config", "password", m_passwd->getLineEditText());
    WriteSettingToIni("config", "autologin", QString("%1").arg(m_autoLoginFlag));
}

void UserFrame::setNextBtnStyle()
{
    if(userSetFinished && hostSetFinished && pwdSetFinished && cpwdSetFinished) {
        m_nextBtn->setEnabled(true);
        m_nextBtn->setStyleSheet("color:#FFFFFF;background:#1D8EFF;");
    } else {
        m_nextBtn->setEnabled(false);
        m_nextBtn->setStyleSheet("color: rgba(255, 255, 255, 0.4);background:#2D6BAA;");
    }
}


//bool UserFrame::eventFilter(QObject *watched, QEvent *event)
//{
//    if( watched->objectName() == "username" || watched->objectName() == "hostname" || watched->objectName() == "confirm_passwd"
//            || watched->objectName() == "passwd") {
//        if( event->type() == QEvent::MouseButtonPress ) {
//        //由于我需要传递参数，但是监控的便签有2个，所以要找到指定的qlineedit
////            QLineEdit *point = this->findChild<QLineEdit *>( watched->objectName() );
//        //处理鼠标点击事件
////            key->show();
//            return true;
//        } else {
//            return false;
//        }
//    }
//    if(watched == m_passwd) {
//        if(event->type() == QEvent::MouseButtonPress) {
//            emit signalToolTip();
//        }
//    }

//    return QWidget::eventFilter(watched,event);
//}
void UserFrame::showToolTip()
{
    //TODO
//    w->setText(tooltipStr);//密码长度不能少于6个字符    密码必须有两种不同字符

//    w->setDerection(ArrowWidget::Derection::left);
//    w->setObjectName("ArrowWidget");
//    w->hide();
}


void UserFrame::keyPressEvent(QKeyEvent *event)
{
    qDebug()<< Q_FUNC_INFO << event->key() << "key pressed!";
    if(event->key() == Qt::Key_Enter || event->key() == Qt::Key_Return)
    {
        emit enterpressed();
    }
    else if(event->key() == Qt::Key_Backspace)
    {
        emit backspacepressed();
    }
    else if(event->key() == Qt::Key_Escape)
    {
        this->setFocus();
    }
    else
        QWidget::keyPressEvent(event);
}

}
