﻿#include "installerrorframe.h"
#include <QFile>
#include <QDebug>
#include <QTextStream>
#include <QProcess>
#include <QEvent>
#include <QMessageBox>
#include "../PluginService/kcommand.h"
#include "../PluginService/ui_unit/xrandrobject.h"
using namespace KServer;
namespace KInstaller {
InstallErrorFrame::InstallErrorFrame(QWidget *parent) : QWidget(parent)
{
    initUI();
    initAllConnect();
    addStyleSheet();
    scpTargetLog();
    this->setFocus();
}

void InstallErrorFrame::scpTargetLog()
{
    QFile file("/target/var/log/installer/kylin-os-installer.log");
    if(file.exists()) {
        KServer::KCommand::getInstance()->RunScripCommand("rsync", {"/target/var/log/installer/kylin-os-installer.log", "/var/log/installer/kylin-os-installer_target.log"});
    }
}

void InstallErrorFrame::initUI()
{
    QGridLayout* gLayout = new QGridLayout;
    gLayout->setContentsMargins(0, 0, 0, 0);
    this->setLayout(gLayout);
    gLayout->setColumnStretch(0, 3);
    gLayout->setColumnStretch(1, 2);
    gLayout->setColumnStretch(2, 3);

    QRect rect = KServer::GetScreenRect();

    QWidget* widget = new QWidget;
    m_mainTitle = new QLabel(widget);
    m_mainTitle->setObjectName("mainTitle");
    m_mainTitle->setAlignment(Qt::AlignLeft);
    gLayout->addItem(new QSpacerItem(10, rect.height() * 0.3, QSizePolicy::Expanding, QSizePolicy::Minimum), 0, 1, 1, 1);
//    gLayout->addWidget(m_mainTitle, 1, 1, 1, 1, Qt::AlignCenter | Qt::Align:Top);
//    gLayout->addItem(new QSpacerItem(10, 40, QSizePolicy::Expanding, QSizePolicy::Minimum), 2, 1, 1, 1);

    m_errorLable = new QLabel(widget);

    m_errorLable->setAlignment(Qt::AlignRight);
    m_errorLable->setPixmap(QPixmap(":/data/svg/failed0.svg"));//TODO
//    gLayout->addWidget(m_errorLable, 3, 1, 1, 1, Qt::AlignCenter | Qt::AlignTop);
//    gLayout->addItem(new QSpacerItem(10, 40, QSizePolicy::Expanding, QSizePolicy::Minimum), 4, 1, 1, 1);

    QHBoxLayout* hlayout = new QHBoxLayout;
    hlayout->setAlignment(Qt::AlignCenter);
    gLayout->addLayout(hlayout, 1, 1, 1, 1, Qt::AlignCenter);
    hlayout->addWidget(m_errorLable);
    hlayout->addWidget(m_mainTitle);
    gLayout->addLayout(hlayout, 1, 1, 1, 1, Qt::AlignCenter);
    gLayout->addLayout(hlayout, 1, 1, 1, 1, Qt::AlignCenter);
//    gLayout->addWidget(m_errorLable, 1, 1, 1, 1, Qt::AlignCenter);
    gLayout->addItem(new QSpacerItem(10, 33, QSizePolicy::Expanding, QSizePolicy::Minimum), 2, 1, 1, 1);
    m_tipInfo = new QLabel;
    m_tipInfo->setObjectName("m_tipInfo");
    gLayout->addWidget(m_tipInfo, 3, 1, 1, 1, Qt::AlignHCenter | Qt::AlignTop);
    gLayout->setRowStretch(4, 3);

    m_restartBtn = new QPushButton;
    m_restartBtn->setObjectName("restartBtn");
    gLayout->addWidget(m_restartBtn, 5, 1, 1, 1, Qt::AlignCenter | Qt::AlignTop);
    gLayout->addItem(new QSpacerItem(10, 120, QSizePolicy::Expanding, QSizePolicy::Minimum), 6, 1, 1, 1);
    translateStr();
}

void InstallErrorFrame::changeEvent(QEvent *event)
{
    if (event->type() == QEvent::LanguageChange) {
        translateStr();
    } else {
        QWidget::changeEvent(event);
    }
}

void InstallErrorFrame::translateStr()
{
    m_tipInfo->setText(tr("The computer restarted unexpectedly or encountered an error.\n"
                         "please click \"restart\" to restart the computer, and then reinstall"));

    m_mainTitle->setText(tr("Installation Failed"));
    m_restartBtn->setText(tr("Restart"));

}

void InstallErrorFrame::addStyleSheet()
{
    QFile file(":/data/qss/InstallErrorFrame.css");
    file.open(QFile::ReadOnly);
    QTextStream filetext(&file);
    QString stylesheet = filetext.readAll();
    file.close();

    this->setStyleSheet(stylesheet);
}

void InstallErrorFrame::initAllConnect()
{
    connect(m_restartBtn, &QPushButton::clicked, this, &InstallErrorFrame::clickRestartBtn);
}

void InstallErrorFrame::clickNextButton()
{

}

void InstallErrorFrame::clickRestartBtn()
{
    QProcess process;
    process.startDetached("systemctl reboot");
}

void InstallErrorFrame::keyPressEvent(QKeyEvent *event)
{
    if(event->key() == Qt::Key_Enter || event->key() == Qt::Key_Return)
        clickRestartBtn();//;
        //this->releaseKeyboard;
    else if(event->key() == Qt::Key_Escape || event->key() == Qt::Key_Control)
        ;
    else
        QWidget::keyPressEvent(event);
}
}


